@@ -28,9 +28,14 @@
  * MT safe
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "glib.h"
 
 
+#ifndef DISABLE_MEM_POOLS
 struct _GAllocator /* from gmem.c */
 {
   gchar         *name;
@@ -160,8 +165,8 @@ g_list_free (GList *list)
           last_node->prev = NULL;
           last_node = last_node->next;
         }
-      last_node->data = NULL
-      last_node->prev = NULL
+      last_node->data = NULL;
+      last_node->prev = NULL;
 #else /* !ENABLE_GC_FRIENDLY */
       list->data = list->next;  
 #endif /* ENABLE_GC_FRIENDLY */
@@ -197,6 +202,41 @@ g_list_free_1 (GList *list)
   _g_list_free_1 (list);
 }
 
+#else /* DISABLE_MEM_POOLS */
+
+#define _g_list_alloc g_list_alloc
+GList*
+g_list_alloc (void)
+{
+  GList *list;
+  
+  list = g_new0 (GList, 1);
+  
+  return list;
+}
+
+void
+g_list_free (GList *list)
+{
+  GList *last;
+  
+  while (list)
+    {
+      last = list;
+      list = list->next;
+      g_free (last);
+    }
+}
+
+#define _g_list_free_1 g_list_free_1
+void
+g_list_free_1 (GList *list)
+{
+  g_free (list);
+}
+
+#endif
+
 GList*
 g_list_append (GList	*list,
 	       gpointer	 data)
@@ -324,6 +364,34 @@ g_list_remove (GList	     *list,
   return list;
 }
 
+GList*
+g_list_remove_all (GList	*list,
+		   gconstpointer data)
+{
+  GList *tmp = list;
+
+  while (tmp)
+    {
+      if (tmp->data != data)
+	tmp = tmp->next;
+      else
+	{
+	  GList *next = tmp->next;
+
+	  if (tmp->prev)
+	    tmp->prev->next = next;
+	  else
+	    list = next;
+	  if (next)
+	    next->prev = tmp->prev;
+
+	  _g_list_free_1 (tmp);
+	  tmp = next;
+	}
+    }
+  return list;
+}
+
 static inline GList*
 _g_list_remove_link (GList *list,
 		     GList *link)
@@ -415,6 +483,16 @@ g_list_nth (GList *list,
   return list;
 }
 
+GList*
+g_list_nth_prev (GList *list,
+		 guint  n)
+{
+  while ((n-- > 0) && list)
+    list = list->prev;
+  
+  return list;
+}
+
 gpointer
 g_list_nth_data (GList     *list,
 		 guint      n)
@@ -596,18 +674,26 @@ g_list_insert_sorted (GList        *list,
 }
 
 static GList *
-g_list_sort_merge (GList       *l1, 
-		   GList       *l2,
-		   GCompareFunc compare_func)
+g_list_sort_merge (GList     *l1, 
+		   GList     *l2,
+		   GFunc     compare_func,
+		   gboolean  use_data,
+		   gpointer  user_data)
 {
   GList list, *l, *lprev;
+  gint cmp;
 
   l = &list; 
   lprev = NULL;
 
   while (l1 && l2)
     {
-      if (compare_func (l1->data, l2->data) < 0)
+      if (use_data)
+	cmp = ((GCompareDataFunc) compare_func) (l1->data, l2->data, user_data);
+      else
+	cmp = ((GCompareFunc) compare_func) (l1->data, l2->data);
+
+      if (cmp <= 0)
         {
 	  l->next = l1;
 	  l = l->next;
@@ -631,8 +717,10 @@ g_list_sort_merge (GList       *l1,
 }
 
 GList* 
-g_list_sort (GList       *list,
-	     GCompareFunc compare_func)
+g_list_sort_real (GList    *list,
+		  GFunc     compare_func,
+		  gboolean  use_data,
+		  gpointer  user_data)
 {
   GList *l1, *l2;
   
@@ -653,9 +741,27 @@ g_list_sort (GList       *list,
   l2 = l1->next; 
   l1->next = NULL; 
 
-  return g_list_sort_merge (g_list_sort (list, compare_func),
-			    g_list_sort (l2,   compare_func),
-			    compare_func);
+  return g_list_sort_merge (g_list_sort_real (list, compare_func, use_data, user_data),
+			    g_list_sort_real (l2, compare_func, use_data, user_data),
+			    compare_func,
+			    use_data,
+			    user_data);
+}
+
+GList *
+g_list_sort (GList        *list,
+	     GCompareFunc  compare_func)
+{
+  return g_list_sort_real (list, (GFunc) compare_func, FALSE, NULL);
+			    
+}
+
+GList *
+g_list_sort_with_data (GList            *list,
+		       GCompareDataFunc  compare_func,
+		       gpointer          user_data)
+{
+  return g_list_sort_real (list, (GFunc) compare_func, TRUE, user_data);
 }
 
 GList* 
@@ -691,7 +797,8 @@ g_list_sort2 (GList       *list,
 	{
 	  dst->data = g_list_sort_merge (src->data,
 					 src->next->data,
-					 compare_func);
+					 (GFunc) compare_func,
+					 FALSE, NULL);
 	  dstprev = dst;
 	  dst = dst->next;
 	  src = src->next->next;
