@@ -91,27 +91,50 @@ static GPrivate* g_log_depth = NULL;
 #ifdef G_OS_WIN32
 #  define STRICT
 #  include <windows.h>
+#  undef STRICT
 #  include <process.h>          /* For _getpid() */
 
 static gboolean alloc_console_called = FALSE;
 
+static gboolean gonna_abort = FALSE;
+
+/* This default message will usually be overwritten. */
+/* Yes, a fixed size buffer is bad. So sue me. But g_error is never
+ * with huge strings, is it? */
+static char fatal_msg_buf[1000] = "Unspecified fatal error encountered, aborting.";
+static char *fatal_msg_ptr = fatal_msg_buf;
+
 /* Just use stdio. If we're out of memory, we're hosed anyway. */
 #undef write
 static inline int
-write (FILE       *fd,
-       const char *buf,
-       int         len)
+dowrite (FILE        *fd,
+	 const void  *buf,
+	 unsigned int len)
 {
+  if (gonna_abort)
+    {
+      memcpy (fatal_msg_ptr, buf, len);
+      fatal_msg_ptr += len;
+      *fatal_msg_ptr = 0;
+      return len;
+    }
+
   fwrite (buf, len, 1, fd);
   fflush (fd);
 
   return len;
 }
+
+#define write(fd, buf, len) dowrite(fd, buf, len)
+
 static void
 ensure_stdout_valid (void)
 {
   HANDLE handle;
 
+  if (gonna_abort)
+    return;
+
   if (!alloc_console_called)
     {
       handle = GetStdHandle (STD_OUTPUT_HANDLE);
@@ -410,6 +433,13 @@ g_logv (const gchar   *log_domain,
 	      else
 		abort ();
 #else /* !G_ENABLE_DEBUG || !SIGTRAP */
+#ifdef G_OS_WIN32
+	      MessageBox (NULL, fatal_msg_buf, NULL, MB_OK);
+#endif
+# if defined (_MSC_VER) && defined (_DEBUG)
+	      /* let's see the call stack ... */
+	      __asm int 3
+# endif
 	      abort ();
 #endif /* !G_ENABLE_DEBUG || !SIGTRAP */
 	    }
@@ -470,6 +500,7 @@ g_log_default_handler (const gchar    *log_domain,
    * DOS prompt.
    */
   fd = stdout;
+  gonna_abort = is_fatal;
 #else
   fd = (log_level > G_LOG_LEVEL_MESSAGE) ? 1 : 2;
 #endif
@@ -853,7 +884,7 @@ printf_string_upper_bound (const gchar *format,
 		  GDoubleIEEE754 u_double;
 		  guint v_uint;
 		  gint v_int;
-		  gchar *v_string;
+		  const gchar *v_string;
 
 		  /* beware of positional parameters
 		   */
