@@ -1,5 +1,5 @@
 /* GObject - GLib Type, Object, Parameter and Signal Library
- * Copyright (C) 1997, 1998, 1999, 2000 Tim Janik and Red Hat, Inc.
+ * Copyright (C) 1997-1999, 2000-2001 Tim Janik and Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,11 +24,7 @@
 
 #include	<gobject/gvalue.h>
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 /* --- type macros --- */
 #define G_TYPE_IS_PARAM(type)		(G_TYPE_FUNDAMENTAL (type) == G_TYPE_PARAM)
@@ -37,24 +33,45 @@ extern "C" {
 #define G_PARAM_SPEC(pspec)		(G_TYPE_CHECK_INSTANCE_CAST ((pspec), G_TYPE_PARAM, GParamSpec))
 #define G_IS_PARAM_SPEC(pspec)		(G_TYPE_CHECK_INSTANCE_TYPE ((pspec), G_TYPE_PARAM))
 #define G_PARAM_SPEC_GET_CLASS(pspec)	(G_TYPE_INSTANCE_GET_CLASS ((pspec), G_TYPE_PARAM, GParamSpecClass))
-#define	G_IS_PARAM_VALUE(pspec, value)	(g_type_is_a (G_VALUE_TYPE (value), G_PARAM_SPEC_VALUE_TYPE (pspec))) /* FIXME */
-#define	G_PARAM_SPEC_VALUE_TYPE(pspec)	(G_PARAM_SPEC_GET_CLASS (pspec)->value_type)
-
+#define	G_PARAM_SPEC_VALUE_TYPE(pspec)	(G_PARAM_SPEC (pspec)->value_type)
+#define G_VALUE_HOLDS_PARAM(value)	(G_TYPE_CHECK_VALUE_TYPE ((value), G_TYPE_PARAM))
+       
 
 /* --- flags --- */
 typedef enum
 {
   G_PARAM_READABLE            = 1 << 0,
   G_PARAM_WRITABLE            = 1 << 1,
-  G_PARAM_MASK                = 0x000f,
-  /* bits in the range 0xfff0 are reserved for 3rd party usage */
-  G_PARAM_USER_MASK           = 0xfff0
+  G_PARAM_CONSTRUCT	      = 1 << 2,
+  G_PARAM_CONSTRUCT_ONLY      = 1 << 3,
+  G_PARAM_LAX_VALIDATION      = 1 << 4,
+  G_PARAM_PRIVATE	      = 1 << 5
 } GParamFlags;
+#define	G_PARAM_READWRITE	(G_PARAM_READABLE | G_PARAM_WRITABLE)
+#define	G_PARAM_MASK		(0x000000ff)
+/* bits in the range 0xffffff00 are reserved for 3rd party usage */
+#define	G_PARAM_USER_SHIFT	(8)
 
 
 /* --- typedefs & structures --- */
-typedef struct _GParamSpecClass GParamSpecClass;
 typedef struct _GParamSpec      GParamSpec;
+typedef struct _GParamSpecClass GParamSpecClass;
+typedef struct _GParamSpecPool  GParamSpecPool;
+struct _GParamSpec
+{
+  GTypeInstance  g_type_instance;
+
+  gchar         *name;
+  gchar         *nick;
+  gchar         *blurb;
+  GParamFlags    flags;
+  GType		 value_type;
+
+  /*< private >*/
+  GType		 owner_type;
+  GData		*qdata;
+  guint          ref_count;
+};
 struct _GParamSpecClass
 {
   GTypeClass      g_type_class;
@@ -72,25 +89,12 @@ struct _GParamSpecClass
 					 const GValue *value1,
 					 const GValue *value2);
 };
-struct _GParamSpec
-{
-  GTypeInstance  g_instance;
-
-  gchar         *name;
-  gchar         *nick;
-  gchar         *blurb;
-  GParamFlags    flags;
-
-  /*< private >*/
-  GType		 owner_type;
-  GData		*qdata;
-  guint          ref_count;
-};
 
 
 /* --- prototypes --- */
 GParamSpec*	g_param_spec_ref		(GParamSpec    *pspec);
 void		g_param_spec_unref		(GParamSpec    *pspec);
+void		g_param_spec_sink		(GParamSpec    *pspec);
 gpointer        g_param_spec_get_qdata		(GParamSpec    *pspec,
 						 GQuark         quark);
 void            g_param_spec_set_qdata		(GParamSpec    *pspec,
@@ -108,9 +112,17 @@ gboolean	g_param_value_defaults		(GParamSpec    *pspec,
 						 GValue	       *value);
 gboolean	g_param_value_validate		(GParamSpec    *pspec,
 						 GValue	       *value);
+gboolean	g_param_value_convert		(GParamSpec    *dest_value_spec,
+						 const GValue  *src_value,
+						 GValue	       *dest_value,
+						 gboolean	strict_validation);
 gint		g_param_values_cmp		(GParamSpec    *pspec,
 						 const GValue  *value1,
 						 const GValue  *value2);
+void            g_value_set_param               (GValue	       *value,
+						 GParamSpec    *param);
+GParamSpec*     g_value_get_param               (const GValue  *value);
+GParamSpec*     g_value_dup_param               (const GValue  *value);
 
 
 /* --- convenience functions --- */
@@ -137,23 +149,25 @@ GType	g_param_type_register_static	(const gchar		  *name,
 					 const GParamSpecTypeInfo *pspec_info);
 
 
-/* --- private --- */
+/* --- protected --- */
 gpointer	g_param_spec_internal		(GType	        param_type,
 						 const gchar   *name,
 						 const gchar   *nick,
 						 const gchar   *blurb,
 						 GParamFlags    flags);
-GHashTable*	g_param_spec_hash_table_new	(void);
-void		g_param_spec_hash_table_insert	(GHashTable    *hash_table,
-						 GParamSpec    *pspec,
-						 GType		owner_type);
-void		g_param_spec_hash_table_remove	(GHashTable    *hash_table,
-						 GParamSpec    *pspec);
-GParamSpec*	g_param_spec_hash_table_lookup	(GHashTable    *hash_table,
-						 const gchar   *param_name,
-						 GType		owner_type,
-						 gboolean       try_ancestors,
-						 const gchar  **trailer);
+GParamSpecPool* g_param_spec_pool_new		(gboolean	type_prefixing);
+void		g_param_spec_pool_insert	(GParamSpecPool	*pool,
+						 GParamSpec	*pspec,
+						 GType		 owner_type);
+void		g_param_spec_pool_remove	(GParamSpecPool	*pool,
+						 GParamSpec	*pspec);
+GParamSpec*	g_param_spec_pool_lookup	(GParamSpecPool	*pool,
+						 const gchar	*param_name,
+						 GType		 owner_type,
+						 gboolean	 walk_ancestors);
+GList*		g_param_spec_pool_list		(GParamSpecPool	*pool,
+						 GType		 owner_type);
+
 
 
 /* contracts:
@@ -172,8 +186,6 @@ GParamSpec*	g_param_spec_hash_table_lookup	(GHashTable    *hash_table,
  *	>0 if value1 > value2, and 0 otherwise (they are equal)
  */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __G_PARAM_H__ */
