@@ -1,5 +1,5 @@
 /* GObject - GLib Type, Object, Parameter and Signal Library
- * Copyright (C) 2000 Red Hat, Inc.
+ * Copyright (C) 2000-2001 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,28 +19,38 @@
  * this code is based on the original GtkSignal implementation
  * for the Gtk+ library by Peter Mattis <petm@xcf.berkeley.edu>
  */
-#include <string.h> 
+
+/*
+ * MT safe
+ */
 
 #include        "gsignal.h"
-
 #include        "gbsearcharray.h"
+#include        "gvaluecollector.h"
+#include	"gvaluetypes.h"
+#include	"gboxed.h"
+#include	<string.h> 
 
 
 /* pre allocation configurations
  */
-#define BSA_PRE_ALLOC           (20)
+#define	MAX_STACK_VALUES	(16)
 #define HANDLER_PRE_ALLOC       (48)
 #define EMISSION_PRE_ALLOC      (16)
 
-#define TIGHT_MEMORY    (1)
-
 #define REPORT_BUG      "please report occourance circumstances to gtk-devel-list@gnome.org"
+#ifdef	G_ENABLE_DEBUG
+#define IF_DEBUG(debug_type, cond)	if ((_g_type_debug_flags & G_TYPE_DEBUG_ ## debug_type) || cond)
+static volatile gpointer *g_trace_instance_signals = NULL;
+static volatile gpointer *g_trap_instance_signals = NULL;
+#endif	/* G_ENABLE_DEBUG */
 
 
 /* --- generic allocation --- */
-/* we can special case allocations generically by replacing
+/* we special case allocations generically by replacing
  * these functions with more speed/memory aware variants
  */
+#ifndef	DISABLE_MEM_POOLS
 static inline gpointer
 g_generic_node_alloc (GTrashStack **trash_stack_p,
                       guint         sizeof_node,
@@ -64,12 +74,11 @@ g_generic_node_alloc (GTrashStack **trash_stack_p,
   
   return node;
 }
-static inline void
-g_generic_node_free (GTrashStack **trash_stack_p,
-                     gpointer      node)
-{
-  g_trash_stack_push (trash_stack_p, node);
-}
+#define	g_generic_node_free(trash_stack_p, node) g_trash_stack_push (trash_stack_p, node)
+#else	/* !DISABLE_MEM_POOLS */
+#define	g_generic_node_alloc(t,sizeof_node,p)	 g_malloc (sizeof_node)
+#define	g_generic_node_free(t,node)		 g_free (node)
+#endif	/* !DISABLE_MEM_POOLS */
 
 
 /* --- typedefs --- */
@@ -131,7 +140,9 @@ static inline Emission*		emission_find		(Emission	 *emission_list,
 							 guint		  signal_id,
 							 GQuark		  detail,
 							 gpointer	  instance);
-static	      void		signal_emit_R		(SignalNode	 *node,
+static gint			signal_key_cmp		(gconstpointer	  node1,
+							 gconstpointer	  node2);
+static	      gboolean		signal_emit_R		(SignalNode	 *node,
 							 GQuark		  detail,
 							 gpointer	  instance,
 							 GValue		 *return_value,
@@ -139,6 +150,18 @@ static	      void		signal_emit_R		(SignalNode	 *node,
 
 
 /* --- structures --- */
+typedef struct
+{
+  GSignalAccumulator func;
+  gpointer           data;
+} SignalAccumulator;
+typedef struct
+{
+  GHook hook;
+  GQuark detail;
+} SignalHook;
+#define	SIGNAL_HOOK(hook)	((SignalHook*) (hook))
+
 struct _SignalNode
 {
   /* permanent portion */
@@ -150,10 +173,10 @@ struct _SignalNode
   /* reinitializable portion */
   guint              flags : 8;
   guint              n_params : 8;
-  GType		    *param_types;
-  GType		     return_type;
+  GType		    *param_types; /* mangled with G_SIGNAL_TYPE_STATIC_SCOPE flag */
+  GType		     return_type; /* mangled with G_SIGNAL_TYPE_STATIC_SCOPE flag */
   GClosure          *class_closure;
-  GSignalAccumulator accumulator;
+  SignalAccumulator *accumulator;
   GSignalCMarshaller c_marshaller;
   GHookList         *emission_hooks;
 };
@@ -204,11 +227,12 @@ struct _HandlerMatch
 
 
 /* --- variables --- */
-static GBSearchArray  g_signal_key_bsa = { NULL, 0, 0, 0, NULL };
+static GBSearchArray  g_signal_key_bsa = G_STATIC_BSEARCH_ARRAY_INIT (sizeof (SignalKey),
+								      signal_key_cmp,
+								      G_BSEARCH_ARRAY_ALIGN_POWER2);
 static GHashTable    *g_handler_list_bsa_ht = NULL;
 static Emission      *g_recursive_emissions = NULL;
 static Emission      *g_restart_emissions = NULL;
-static GTrashStack   *g_bsa_ts = NULL;
 static GTrashStack   *g_handler_ts = NULL;
 static GTrashStack   *g_emission_ts = NULL;
 G_LOCK_DEFINE_STATIC (g_signal_mutex);
@@ -233,21 +257,43 @@ static inline guint
 signal_id_lookup (GQuark quark,
 		  GType  itype)
 {
+  GType *ifaces, type = itype;
+  SignalKey key;
+  guint n_ifaces;
+
+  key.quark = quark;
+
+  /* try looking up signals for this type and its anchestors */
   do
     {
-      SignalKey key, *signal_key;
-      
-      key.itype = itype;
-      key.quark = quark;
+      SignalKey *signal_key;
       
+      key.itype = type;
       signal_key = g_bsearch_array_lookup (&g_signal_key_bsa, &key);
       
       if (signal_key)
 	return signal_key->signal_id;
       
-      itype = g_type_parent (itype);
+      type = g_type_parent (type);
     }
-  while (itype);
+  while (type);
+
+  /* no luck, try interfaces it exports */
+  ifaces = g_type_interfaces (itype, &n_ifaces);
+  while (n_ifaces--)
+    {
+      SignalKey *signal_key;
+
+      key.itype = ifaces[n_ifaces];
+      signal_key = g_bsearch_array_lookup (&g_signal_key_bsa, &key);
+
+      if (signal_key)
+	{
+	  g_free (ifaces);
+	  return signal_key->signal_id;
+	}
+    }
+  g_free (ifaces);
   
   return 0;
 }
@@ -270,14 +316,7 @@ handler_list_ensure (guint    signal_id,
   
   if (!hlbsa)
     {
-      hlbsa = g_generic_node_alloc (&g_bsa_ts,
-                                    sizeof (GBSearchArray),
-                                    BSA_PRE_ALLOC);
-      hlbsa->cmp_func = handler_lists_cmp;
-      hlbsa->sizeof_node = sizeof (HandlerList);
-      hlbsa->flags = G_BSEARCH_DEFER_SHRINK;
-      hlbsa->n_nodes = 0;
-      hlbsa->nodes = NULL;
+      hlbsa = g_bsearch_array_new (sizeof (HandlerList), handler_lists_cmp, G_BSEARCH_ARRAY_DEFER_SHRINK);
       g_hash_table_insert (g_handler_list_bsa_ht, instance, hlbsa);
     }
   key.signal_id = signal_id;
@@ -376,7 +415,7 @@ handlers_find (gpointer         instance,
     {
       HandlerList *hlist = handler_list_lookup (signal_id, instance);
       Handler *handler;
-      SignalNode *node;
+      SignalNode *node = NULL;
       
       if (mask & G_SIGNAL_MATCH_FUNC)
 	{
@@ -413,7 +452,7 @@ handlers_find (gpointer         instance,
           for (i = 0; i < hlbsa->n_nodes; i++)
             {
               HandlerList *hlist = g_bsearch_array_get_nth (hlbsa, i);
-	      SignalNode *node;
+	      SignalNode *node = NULL;
               Handler *handler;
               
 	      if (!(mask & G_SIGNAL_MATCH_FUNC))
@@ -515,7 +554,7 @@ handler_insert (guint    signal_id,
 {
   HandlerList *hlist;
   
-  g_assert (handler->prev == NULL && handler->next == NULL); // FIXME: paranoid
+  g_assert (handler->prev == NULL && handler->next == NULL); /* paranoid */
   
   hlist = handler_list_ensure (signal_id, instance);
   if (!hlist->handlers)
@@ -614,11 +653,6 @@ g_signal_init (void) /* sync with gtype.c */
       /* handler_id_node_prepend() requires this */
       g_assert (sizeof (GList) == sizeof (HandlerMatch));
       
-      /* setup signal key array */
-      g_signal_key_bsa.cmp_func = signal_key_cmp;
-      g_signal_key_bsa.sizeof_node = sizeof (SignalKey);
-      g_signal_key_bsa.flags = 0; /* alloc-only */
-      
       /* setup handler list binary searchable array hash table (in german, that'd be one word ;) */
       g_handler_list_bsa_ht = g_hash_table_new (g_direct_hash, NULL);
       
@@ -671,7 +705,7 @@ g_signal_stop_emission (gpointer instance,
       G_UNLOCK (g_signal_mutex);
       return;
     }
-  if (node && g_type_conforms_to (G_TYPE_FROM_INSTANCE (instance), node->itype))
+  if (node && g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
     {
       Emission *emission_list = node->flags & G_SIGNAL_NO_RECURSE ? g_restart_emissions : g_recursive_emissions;
       Emission *emission = emission_find (emission_list, signal_id, detail, instance);
@@ -693,6 +727,88 @@ g_signal_stop_emission (gpointer instance,
   G_UNLOCK (g_signal_mutex);
 }
 
+static void
+signal_finalize_hook (GHookList *hook_list,
+		      GHook     *hook)
+{
+  GDestroyNotify destroy = hook->destroy;
+
+  if (destroy)
+    {
+      hook->destroy = NULL;
+      G_UNLOCK (g_signal_mutex);
+      destroy (hook->data);
+      G_LOCK (g_signal_mutex);
+    }
+}
+
+guint
+g_signal_add_emission_hook (guint               signal_id,
+			    GQuark              detail,
+			    GSignalEmissionHook hook_func,
+			    gpointer            hook_data,
+			    GDestroyNotify      data_destroy)
+{
+  static guint seq_hook_id = 1;
+  SignalNode *node;
+  GHook *hook;
+  SignalHook *signal_hook;
+
+  g_return_val_if_fail (signal_id > 0, 0);
+  g_return_val_if_fail (hook_func != NULL, 0);
+
+  G_LOCK (g_signal_mutex);
+  node = LOOKUP_SIGNAL_NODE (signal_id);
+  if (!node || node->destroyed || (node->flags & G_SIGNAL_NO_HOOKS))
+    {
+      g_warning ("%s: invalid signal id `%u'", G_STRLOC, signal_id);
+      G_UNLOCK (g_signal_mutex);
+      return 0;
+    }
+  if (detail && !(node->flags & G_SIGNAL_DETAILED))
+    {
+      g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
+      G_UNLOCK (g_signal_mutex);
+      return 0;
+    }
+  if (!node->emission_hooks)
+    {
+      node->emission_hooks = g_new (GHookList, 1);
+      g_hook_list_init (node->emission_hooks, sizeof (SignalHook));
+      node->emission_hooks->finalize_hook = signal_finalize_hook;
+    }
+  hook = g_hook_alloc (node->emission_hooks);
+  hook->data = hook_data;
+  hook->func = hook_func;
+  hook->destroy = data_destroy;
+  signal_hook = SIGNAL_HOOK (hook);
+  signal_hook->detail = detail;
+  node->emission_hooks->seq_id = seq_hook_id;
+  g_hook_append (node->emission_hooks, hook);
+  seq_hook_id = node->emission_hooks->seq_id;
+  G_UNLOCK (g_signal_mutex);
+
+  return hook->hook_id;
+}
+
+void
+g_signal_remove_emission_hook (guint signal_id,
+			       guint hook_id)
+{
+  SignalNode *node;
+
+  g_return_if_fail (signal_id > 0);
+  g_return_if_fail (hook_id > 0);
+
+  G_LOCK (g_signal_mutex);
+  node = LOOKUP_SIGNAL_NODE (signal_id);
+  if (!node || node->destroyed)
+    g_warning ("%s: invalid signal id `%u'", G_STRLOC, signal_id);
+  else if (!node->emission_hooks || !g_hook_destroy (node->emission_hooks, hook_id))
+    g_warning ("%s: signal \"%s\" had no hook (%u) to remove", G_STRLOC, node->name, hook_id);
+  G_UNLOCK (g_signal_mutex);
+}
+
 static inline guint
 signal_parse_name (const gchar *name,
 		   GType        itype,
@@ -744,6 +860,7 @@ g_signal_parse_name (const gchar *detailed_signal,
 		     GQuark      *detail_p,
 		     gboolean	  force_detail_quark)
 {
+  SignalNode *node;
   GQuark detail = 0;
   guint signal_id;
   
@@ -753,18 +870,18 @@ g_signal_parse_name (const gchar *detailed_signal,
   G_LOCK (g_signal_mutex);
   signal_id = signal_parse_name (detailed_signal, itype, &detail, force_detail_quark);
   G_UNLOCK (g_signal_mutex);
-  
-  if (signal_id)
-    {
-      if (signal_id_p)
-	*signal_id_p = signal_id;
-      if (detail_p)
-	*detail_p = detail;
-      
-      return TRUE;
-    }
-  else
+
+  node = signal_id ? LOOKUP_SIGNAL_NODE (signal_id) : NULL;
+  if (!node || node->destroyed ||
+      (detail && !(node->flags & G_SIGNAL_DETAILED)))
     return FALSE;
+
+  if (signal_id_p)
+    *signal_id_p = signal_id;
+  if (detail_p)
+    *detail_p = detail;
+  
+  return TRUE;
 }
 
 guint
@@ -783,7 +900,7 @@ g_signal_lookup (const gchar *name,
   return signal_id;
 }
 
-gchar*
+G_CONST_RETURN gchar*
 g_signal_name (guint signal_id)
 {
   SignalNode *node;
@@ -843,7 +960,7 @@ g_signal_list_ids (GType  itype,
   for (i = 0; i < n_nodes; i++)
     if (keys[i].itype == itype)
       {
-	gchar *name = g_quark_to_string (keys[i].quark);
+	const gchar *name = g_quark_to_string (keys[i].quark);
 	
 	/* Signal names with "_" in them are aliases to the same
 	 * name with "-" instead of "_".
@@ -856,7 +973,70 @@ g_signal_list_ids (GType  itype,
   
   G_UNLOCK (g_signal_mutex);
   
-  return (guint *) g_array_free (result, FALSE);
+  return (guint*) g_array_free (result, FALSE);
+}
+
+guint
+g_signal_new_valist (const gchar       *signal_name,
+                     GType              itype,
+                     GSignalFlags       signal_flags,
+                     GClosure          *class_closure,
+                     GSignalAccumulator accumulator,
+		     gpointer		accu_data,
+                     GSignalCMarshaller c_marshaller,
+                     GType              return_type,
+                     guint              n_params,
+                     va_list            args)
+{
+  GType *param_types;
+  guint i;
+  guint signal_id;
+
+  if (n_params > 0)
+    {
+      param_types = g_new (GType, n_params);
+
+      for (i = 0; i < n_params; i++)
+	param_types[i] = va_arg (args, GType);
+    }
+  else
+    param_types = NULL;
+
+  signal_id = g_signal_newv (signal_name, itype, signal_flags,
+			     class_closure, accumulator, accu_data, c_marshaller,
+			     return_type, n_params, param_types);
+  g_free (param_types);
+
+  return signal_id;
+}
+
+guint
+g_signal_newc (const gchar	 *signal_name,
+               GType		  itype,
+               GSignalFlags	  signal_flags,
+               guint              class_offset,
+	       GSignalAccumulator accumulator,
+	       gpointer		  accu_data,
+               GSignalCMarshaller c_marshaller,
+               GType		  return_type,
+               guint		  n_params,
+               ...)
+{
+  va_list args;
+  guint signal_id;
+
+  g_return_val_if_fail (signal_name != NULL, 0);
+  
+  va_start (args, n_params);
+
+  signal_id = g_signal_new_valist (signal_name, itype, signal_flags,
+                                   class_offset ? g_signal_type_cclosure_new (itype, class_offset) : NULL,
+				   accumulator, accu_data, c_marshaller,
+                                   return_type, n_params, args);
+
+  va_end (args);
+ 
+  return signal_id;
 }
 
 guint
@@ -865,6 +1045,7 @@ g_signal_newv (const gchar       *signal_name,
                GSignalFlags       signal_flags,
                GClosure          *class_closure,
                GSignalAccumulator accumulator,
+	       gpointer		  accu_data,
                GSignalCMarshaller c_marshaller,
                GType		  return_type,
                guint              n_params,
@@ -878,9 +1059,12 @@ g_signal_newv (const gchar       *signal_name,
   g_return_val_if_fail (G_TYPE_IS_INSTANTIATABLE (itype) || G_TYPE_IS_INTERFACE (itype), 0);
   if (n_params)
     g_return_val_if_fail (param_types != NULL, 0);
-  if (return_type != G_TYPE_NONE)
+  g_return_val_if_fail ((return_type & G_SIGNAL_TYPE_STATIC_SCOPE) == 0, 0);
+  if (return_type == (G_TYPE_NONE & ~G_SIGNAL_TYPE_STATIC_SCOPE))
     g_return_val_if_fail (accumulator == NULL, 0);
-  
+  if (!accumulator)
+    g_return_val_if_fail (accu_data == NULL, 0);
+
   name = g_strdup (signal_name);
   g_strdelimit (name, G_STR_DELIMITERS ":^", '_');  // FIXME do character checks like for types
   
@@ -909,19 +1093,28 @@ g_signal_newv (const gchar       *signal_name,
       return 0;
     }
   for (i = 0; i < n_params; i++)
-    if (!G_TYPE_IS_VALUE (param_types[i]) ||
-	param_types[i] == G_TYPE_ENUM || param_types[i] == G_TYPE_FLAGS) /* FIXME: kludge */
+    if (!G_TYPE_IS_VALUE (param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE))
       {
 	g_warning (G_STRLOC ": parameter %d of type `%s' for signal \"%s::%s\" is not a value type",
-		   i + 1, g_type_name (param_types[i]), g_type_name (itype), name);
+		   i + 1, g_type_name (param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE), g_type_name (itype), name);
 	g_free (name);
 	G_UNLOCK (g_signal_mutex);
 	return 0;
       }
-  if (return_type != G_TYPE_NONE && !G_TYPE_IS_VALUE (return_type))
+  if (return_type != G_TYPE_NONE && !G_TYPE_IS_VALUE (return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE))
     {
       g_warning (G_STRLOC ": return value of type `%s' for signal \"%s::%s\" is not a value type",
-		 g_type_name (param_types[i]), g_type_name (itype), name);
+		 g_type_name (return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE), g_type_name (itype), name);
+      g_free (name);
+      G_UNLOCK (g_signal_mutex);
+      return 0;
+    }
+  if (return_type != G_TYPE_NONE &&
+      (signal_flags & (G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP)) == G_SIGNAL_RUN_FIRST)
+    {
+      g_warning (G_STRLOC ": signal \"%s::%s\" has return type `%s' and is only G_SIGNAL_RUN_FIRST",
+		 g_type_name (itype), name,
+		 g_type_name (return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE));
       g_free (name);
       G_UNLOCK (g_signal_mutex);
       return 0;
@@ -955,12 +1148,20 @@ g_signal_newv (const gchar       *signal_name,
   node->param_types = g_memdup (param_types, sizeof (GType) * n_params);
   node->return_type = return_type;
   node->class_closure = class_closure ? g_closure_ref (class_closure) : NULL;
-  node->accumulator = accumulator;
+  if (class_closure)
+    g_closure_sink (class_closure);
+  if (accumulator)
+    {
+      node->accumulator = g_new (SignalAccumulator, 1);
+      node->accumulator->func = accumulator;
+      node->accumulator->data = accu_data;
+    }
+  else
+    node->accumulator = NULL;
   node->c_marshaller = c_marshaller;
   node->emission_hooks = NULL;
   if (node->c_marshaller && class_closure && G_CLOSURE_NEEDS_MARSHAL (class_closure))
     g_closure_set_marshal (class_closure, node->c_marshaller);
-  
   G_UNLOCK (g_signal_mutex);
   return signal_id;
 }
@@ -981,7 +1182,7 @@ signal_destroy_R (SignalNode *signal_node)
   signal_node->c_marshaller = NULL;
   signal_node->emission_hooks = NULL;
   
-#ifndef G_DISABLE_CHECKS
+#ifdef	G_ENABLE_DEBUG
   /* check current emissions */
   {
     Emission *emission;
@@ -999,6 +1200,7 @@ signal_destroy_R (SignalNode *signal_node)
   G_UNLOCK (g_signal_mutex);
   g_free (node.param_types);
   g_closure_unref (node.class_closure);
+  g_free (node.accumulator);
   if (node.emission_hooks)
     {
       g_hook_list_clear (node.emission_hooks);
@@ -1023,22 +1225,24 @@ g_signal_connect_closure_by_id (gpointer  instance,
   
   G_LOCK (g_signal_mutex);
   node = LOOKUP_SIGNAL_NODE (signal_id);
-  if (node && detail && !(node->flags & G_SIGNAL_DETAILED))
+  if (node)
     {
-      g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
-      G_UNLOCK (g_signal_mutex);
-      return 0;
-    }
-  if (node && g_type_conforms_to (G_TYPE_FROM_INSTANCE (instance), node->itype))
-    {
-      Handler *handler = handler_new (after);
-      
-      handler_id = handler->id;
-      handler->detail = detail;
-      handler->closure = g_closure_ref (closure);
-      handler_insert (signal_id, instance, handler);
-      if (node->c_marshaller && G_CLOSURE_NEEDS_MARSHAL (closure))
-	g_closure_set_marshal (closure, node->c_marshaller);
+      if (detail && !(node->flags & G_SIGNAL_DETAILED))
+	g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
+      else if (!g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
+	g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
+      else
+	{
+	  Handler *handler = handler_new (after);
+	  
+	  handler_id = handler->id;
+	  handler->detail = detail;
+	  handler->closure = g_closure_ref (closure);
+	  g_closure_sink (closure);
+	  handler_insert (signal_id, instance, handler);
+	  if (node->c_marshaller && G_CLOSURE_NEEDS_MARSHAL (closure))
+	    g_closure_set_marshal (closure, node->c_marshaller);
+	}
     }
   else
     g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
@@ -1047,6 +1251,99 @@ g_signal_connect_closure_by_id (gpointer  instance,
   return handler_id;
 }
 
+guint
+g_signal_connect_closure (gpointer     instance,
+			  const gchar *detailed_signal,
+			  GClosure    *closure,
+			  gboolean     after)
+{
+  guint signal_id, handler_id = 0;
+  GQuark detail = 0;
+  GType itype;
+
+  g_return_val_if_fail (G_TYPE_CHECK_INSTANCE (instance), 0);
+  g_return_val_if_fail (detailed_signal != NULL, 0);
+  g_return_val_if_fail (closure != NULL, 0);
+
+  G_LOCK (g_signal_mutex);
+  itype = G_TYPE_FROM_INSTANCE (instance);
+  signal_id = signal_parse_name (detailed_signal, itype, &detail, TRUE);
+  if (signal_id)
+    {
+      SignalNode *node = LOOKUP_SIGNAL_NODE (signal_id);
+
+      if (detail && !(node->flags & G_SIGNAL_DETAILED))
+	g_warning ("%s: signal `%s' does not support details", G_STRLOC, detailed_signal);
+      else if (!g_type_is_a (itype, node->itype))
+	g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+      else
+	{
+	  Handler *handler = handler_new (after);
+
+	  handler_id = handler->id;
+	  handler->detail = detail;
+	  handler->closure = g_closure_ref (closure);
+	  g_closure_sink (closure);
+	  handler_insert (signal_id, instance, handler);
+	  if (node->c_marshaller && G_CLOSURE_NEEDS_MARSHAL (handler->closure))
+	    g_closure_set_marshal (handler->closure, node->c_marshaller);
+	}
+    }
+  else
+    g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+  G_UNLOCK (g_signal_mutex);
+
+  return handler_id;
+}
+
+guint
+g_signal_connect_data (gpointer       instance,
+		       const gchar   *detailed_signal,
+		       GCallback      c_handler,
+		       gpointer       data,
+		       GClosureNotify destroy_data,
+		       gboolean       swapped,
+		       gboolean       after)
+{
+  guint signal_id, handler_id = 0;
+  GQuark detail = 0;
+  GType itype;
+
+  g_return_val_if_fail (G_TYPE_CHECK_INSTANCE (instance), 0);
+  g_return_val_if_fail (detailed_signal != NULL, 0);
+  g_return_val_if_fail (c_handler != NULL, 0);
+
+  G_LOCK (g_signal_mutex);
+  itype = G_TYPE_FROM_INSTANCE (instance);
+  signal_id = signal_parse_name (detailed_signal, itype, &detail, TRUE);
+  if (signal_id)
+    {
+      SignalNode *node = LOOKUP_SIGNAL_NODE (signal_id);
+
+      if (detail && !(node->flags & G_SIGNAL_DETAILED))
+	g_warning ("%s: signal `%s' does not support details", G_STRLOC, detailed_signal);
+      else if (!g_type_is_a (itype, node->itype))
+	g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+      else
+	{
+	  Handler *handler = handler_new (after);
+
+	  handler_id = handler->id;
+	  handler->detail = detail;
+	  handler->closure = g_closure_ref ((swapped ? g_cclosure_new_swap : g_cclosure_new) (c_handler, data, destroy_data));
+	  g_closure_sink (handler->closure);
+	  handler_insert (signal_id, instance, handler);
+	  if (node->c_marshaller && G_CLOSURE_NEEDS_MARSHAL (handler->closure))
+	    g_closure_set_marshal (handler->closure, node->c_marshaller);
+	}
+    }
+  else
+    g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+  G_UNLOCK (g_signal_mutex);
+
+  return handler_id;
+}
+
 void
 g_signal_handler_block (gpointer instance,
                         guint    handler_id)
@@ -1155,8 +1452,7 @@ g_signal_handlers_destroy (gpointer instance)
 		}
             }
         }
-      g_free (hlbsa->nodes);
-      g_generic_node_free (&g_bsa_ts, hlbsa);
+      g_bsearch_array_destroy (hlbsa);
     }
   G_UNLOCK (g_signal_mutex);
 }
@@ -1342,27 +1638,27 @@ g_signal_emitv (const GValue *instance_and_params,
 		GQuark	      detail,
 		GValue       *return_value)
 {
-  SignalNode *node;
-  gpointer instance;
   const GValue *param_values;
+  gpointer instance;
+  SignalNode *node;
   guint i;
   
   g_return_if_fail (instance_and_params != NULL);
-  instance = g_value_get_as_pointer (instance_and_params);
+  instance = g_value_peek_pointer (instance_and_params);
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (signal_id > 0);
-  
+
   param_values = instance_and_params + 1;
   
   G_LOCK (g_signal_mutex);
   node = LOOKUP_SIGNAL_NODE (signal_id);
-#ifndef G_DISABLE_CHECKS
-  if (!node || !g_type_conforms_to (G_TYPE_FROM_INSTANCE (instance), node->itype))
+  if (!node || !g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
     {
       g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
       G_UNLOCK (g_signal_mutex);
       return;
     }
+#ifdef G_ENABLE_DEBUG
   if (detail && !(node->flags & G_SIGNAL_DETAILED))
     {
       g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
@@ -1370,10 +1666,11 @@ g_signal_emitv (const GValue *instance_and_params,
       return;
     }
   for (i = 0; i < node->n_params; i++)
-    if (!G_VALUE_HOLDS (param_values + i, node->param_types[i]))
+    if (!G_TYPE_CHECK_VALUE_TYPE (param_values + i, node->param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE))
       {
-	g_critical (G_STRLOC ": value for `%s' parameter %u for signal \"%s\" is of type `%s'",
-		    g_type_name (node->param_types[i]),
+	g_critical ("%s: value for `%s' parameter %u for signal \"%s\" is of type `%s'",
+		    G_STRLOC,
+		    g_type_name (node->param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE),
 		    i,
 		    node->name,
 		    G_VALUE_TYPE_NAME (param_values + i));
@@ -1384,16 +1681,18 @@ g_signal_emitv (const GValue *instance_and_params,
     {
       if (!return_value)
 	{
-	  g_critical (G_STRLOC ": return value `%s' for signal \"%s\" is (NULL)",
-		      g_type_name (node->return_type),
+	  g_critical ("%s: return value `%s' for signal \"%s\" is (NULL)",
+		      G_STRLOC,
+		      g_type_name (node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE),
 		      node->name);
 	  G_UNLOCK (g_signal_mutex);
 	  return;
 	}
-      else if (!node->accumulator && !G_VALUE_HOLDS (return_value, node->return_type))
+      else if (!node->accumulator && !G_TYPE_CHECK_VALUE_TYPE (return_value, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE))
 	{
-	  g_critical (G_STRLOC ": return value `%s' for signal \"%s\" is of type `%s'",
-		      g_type_name (node->return_type),
+	  g_critical ("%s: return value `%s' for signal \"%s\" is of type `%s'",
+		      G_STRLOC,
+		      g_type_name (node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE),
 		      node->name,
 		      G_VALUE_TYPE_NAME (return_value));
 	  G_UNLOCK (g_signal_mutex);
@@ -1402,29 +1701,198 @@ g_signal_emitv (const GValue *instance_and_params,
     }
   else
     return_value = NULL;
-#endif	/* !G_DISABLE_CHECKS */
-  
+#endif	/* G_ENABLE_DEBUG */
+
   signal_emit_R (node, detail, instance, return_value, instance_and_params);
-  
   G_UNLOCK (g_signal_mutex);
 }
 
-static void
+void
+g_signal_emit_valist (gpointer instance,
+		      guint    signal_id,
+		      GQuark   detail,
+		      va_list  var_args)
+{
+  GValue *instance_and_params, stack_values[MAX_STACK_VALUES], *free_me = NULL;
+  GValue *param_values;
+  SignalNode *node;
+  guint i;
+
+  g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
+  g_return_if_fail (signal_id > 0);
+
+  G_LOCK (g_signal_mutex);
+  node = LOOKUP_SIGNAL_NODE (signal_id);
+  if (!node || !g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
+    {
+      g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
+      G_UNLOCK (g_signal_mutex);
+      return;
+    }
+#ifndef G_DISABLE_CHECKS
+  if (detail && !(node->flags & G_SIGNAL_DETAILED))
+    {
+      g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
+      G_UNLOCK (g_signal_mutex);
+      return;
+    }
+#endif  /* !G_DISABLE_CHECKS */
+
+  if (node->n_params < MAX_STACK_VALUES)
+    instance_and_params = stack_values;
+  else
+    {
+      free_me = g_new (GValue, node->n_params + 1);
+      instance_and_params = free_me;
+    }
+  param_values = instance_and_params + 1;
+  for (i = 0; i < node->n_params; i++)
+    {
+      gchar *error;
+
+      param_values[i].g_type = 0;
+      g_value_init (param_values + i, node->param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE);
+      G_VALUE_COLLECT (param_values + i,
+		       var_args,
+		       node->param_types[i] & G_SIGNAL_TYPE_STATIC_SCOPE ? G_VALUE_NOCOPY_CONTENTS : 0,
+		       &error);
+      if (error)
+	{
+	  g_warning ("%s: %s", G_STRLOC, error);
+	  g_free (error);
+
+	  /* we purposely leak the value here, it might not be
+	   * in a sane state if an error condition occoured
+	   */
+	  while (i--)
+	    g_value_unset (param_values + i);
+
+	  G_UNLOCK (g_signal_mutex);
+	  g_free (free_me);
+	  return;
+	}
+    }
+  instance_and_params->g_type = 0;
+  g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (instance));
+  g_value_set_instance (instance_and_params, instance);
+  if (node->return_type == G_TYPE_NONE)
+    signal_emit_R (node, detail, instance, NULL, instance_and_params);
+  else
+    {
+      GValue return_value = { 0, };
+      gchar *error = NULL;
+
+      g_value_init (&return_value, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
+      signal_emit_R (node, detail, instance, &return_value, instance_and_params);
+      G_VALUE_LCOPY (&return_value,
+		     var_args,
+		     node->return_type & G_SIGNAL_TYPE_STATIC_SCOPE ? G_VALUE_NOCOPY_CONTENTS : 0,
+		     &error);
+      if (!error)
+	g_value_unset (&return_value);
+      else
+	{
+	  g_warning ("%s: %s", G_STRLOC, error);
+	  g_free (error);
+
+	  /* we purposely leak the value here, it might not be
+	   * in a sane state if an error condition occoured
+	   */
+	}
+    }
+  for (i = 0; i < node->n_params; i++)
+    g_value_unset (param_values + i);
+  g_value_unset (instance_and_params);
+  if (free_me)
+    g_free (free_me);
+  G_UNLOCK (g_signal_mutex);
+}
+
+void
+g_signal_emit (gpointer instance,
+	       guint    signal_id,
+	       GQuark   detail,
+	       ...)
+{
+  va_list var_args;
+
+  va_start (var_args, detail);
+  g_signal_emit_valist (instance, signal_id, detail, var_args);
+  va_end (var_args);
+}
+
+void
+g_signal_emit_by_name (gpointer     instance,
+		       const gchar *detailed_signal,
+		       ...)
+{
+  GQuark detail = 0;
+  guint signal_id;
+
+  g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
+  g_return_if_fail (detailed_signal != NULL);
+
+  G_LOCK (g_signal_mutex);
+  signal_id = signal_parse_name (detailed_signal, G_TYPE_FROM_INSTANCE (instance), &detail, TRUE);
+  G_UNLOCK (g_signal_mutex);
+
+  if (signal_id)
+    {
+      va_list var_args;
+
+      va_start (var_args, detailed_signal);
+      g_signal_emit_valist (instance, signal_id, detail, var_args);
+      va_end (var_args);
+    }
+  else
+    g_warning ("%s: signal name `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+}
+
+static inline gboolean
+accumulate (GSignalInvocationHint *ihint,
+	    GValue                *return_accu,
+	    GValue	          *handler_return,
+	    SignalAccumulator     *accumulator)
+{
+  gboolean continue_emission;
+
+  if (!accumulator)
+    return TRUE;
+
+  continue_emission = accumulator->func (ihint, return_accu, handler_return, accumulator->data);
+  g_value_reset (handler_return);
+
+  return continue_emission;
+}
+
+static gboolean
 signal_emit_R (SignalNode   *node,
 	       GQuark	     detail,
 	       gpointer      instance,
-	       GValue	    *return_value,
+	       GValue	    *emission_return,
 	       const GValue *instance_and_params)
 {
   EmissionState emission_state = 0;
-  GSignalAccumulator accumulator;
+  SignalAccumulator *accumulator;
   GSignalInvocationHint ihint;
   GClosure *class_closure;
   HandlerList *hlist;
   Handler *handler_list = NULL;
-  GValue accu;
-  gboolean accu_used = FALSE;
+  GValue *return_accu, accu = { 0, };
   guint signal_id = node->signal_id;
+  gboolean return_value_altered = FALSE;
+  
+#ifdef	G_ENABLE_DEBUG
+  IF_DEBUG (SIGNALS, g_trace_instance_signals == instance || g_trap_instance_signals == instance)
+    {
+      g_message ("%s::%s(%u) emitted (instance=%p, signal-node=%p)",
+		 g_type_name (G_TYPE_FROM_INSTANCE (instance)),
+		 node->name, detail,
+		 instance, node);
+      if (g_trap_instance_signals == instance)
+	G_BREAKPOINT ();
+    }
+#endif	/* G_ENABLE_DEBUG */
   
   if (node->flags & G_SIGNAL_NO_RECURSE)
     {
@@ -1433,7 +1901,7 @@ signal_emit_R (SignalNode   *node,
       if (emission)
 	{
 	  *emission->state_p = EMISSION_RESTART;
-	  return;
+	  return return_value_altered;
 	}
     }
   ihint.signal_id = node->signal_id;
@@ -1441,10 +1909,11 @@ signal_emit_R (SignalNode   *node,
   accumulator = node->accumulator;
   if (accumulator)
     {
-      G_UNLOCK (g_signal_mutex);
-      g_value_init (&accu, node->return_type);
-      G_LOCK (g_signal_mutex);
+      g_value_init (&accu, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
+      return_accu = &accu;
     }
+  else
+    return_accu = emission_return;
   emission_push ((node->flags & G_SIGNAL_NO_RECURSE) ? &g_restart_emissions : &g_recursive_emissions,
 		 signal_id, detail, instance, &emission_state);
   class_closure = node->class_closure;
@@ -1465,27 +1934,16 @@ signal_emit_R (SignalNode   *node,
       emission_state = EMISSION_RUN;
       
       G_UNLOCK (g_signal_mutex);
-      if (accumulator)
-	{
-	  if (accu_used)
-	    g_value_reset (&accu);
-	  g_closure_invoke (class_closure,
-			    &accu,
-			    node->n_params + 1,
-			    instance_and_params,
-			    &ihint);
-	  if (!accumulator (&ihint, return_value, &accu) &&
-	      emission_state == EMISSION_RUN)
-	    emission_state = EMISSION_STOP;
-	  accu_used = TRUE;
-	}
-      else
-	g_closure_invoke (class_closure,
-			  return_value,
-			  node->n_params + 1,
-			  instance_and_params,
-			  &ihint);
+      g_closure_invoke (class_closure,
+			return_accu,
+			node->n_params + 1,
+			instance_and_params,
+			&ihint);
+      if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
+	  emission_state == EMISSION_RUN)
+	emission_state = EMISSION_STOP;
       G_LOCK (g_signal_mutex);
+      return_value_altered = TRUE;
       
       if (emission_state == EMISSION_STOP)
 	goto EMIT_CLEANUP;
@@ -1495,11 +1953,31 @@ signal_emit_R (SignalNode   *node,
   
   if (node->emission_hooks)
     {
+      gboolean need_destroy, was_in_call, may_recurse = TRUE;
+      GHook *hook;
+
       emission_state = EMISSION_HOOK;
-      
-      G_UNLOCK (g_signal_mutex);
-      g_print ("emission_hooks()\n");
-      G_LOCK (g_signal_mutex);
+      hook = g_hook_first_valid (node->emission_hooks, may_recurse);
+      while (hook)
+	{
+	  SignalHook *signal_hook = SIGNAL_HOOK (hook);
+	  
+	  if (!signal_hook->detail || signal_hook->detail == detail)
+	    {
+	      GSignalEmissionHook hook_func = hook->func;
+	      
+	      was_in_call = G_HOOK_IN_CALL (hook);
+	      hook->flags |= G_HOOK_FLAG_IN_CALL;
+              G_UNLOCK (g_signal_mutex);
+	      need_destroy = !hook_func (&ihint, node->n_params + 1, instance_and_params, hook->data);
+	      G_LOCK (g_signal_mutex);
+	      if (!was_in_call)
+		hook->flags &= ~G_HOOK_FLAG_IN_CALL;
+	      if (need_destroy)
+		g_hook_destroy_link (node->emission_hooks, hook);
+	    }
+	  hook = g_hook_next_valid (node->emission_hooks, hook, may_recurse);
+	}
       
       if (emission_state == EMISSION_RESTART)
 	goto EMIT_RESTART;
@@ -1524,27 +2002,16 @@ signal_emit_R (SignalNode   *node,
 	  else if (!handler->block_count && (!handler->detail || handler->detail == detail))
 	    {
 	      G_UNLOCK (g_signal_mutex);
-	      if (accumulator)
-		{
-		  if (accu_used)
-		    g_value_reset (&accu);
-		  g_closure_invoke (handler->closure,
-				    &accu,
-				    node->n_params + 1,
-				    instance_and_params,
-				    &ihint);
-		  if (!accumulator (&ihint, return_value, &accu) &&
-		      emission_state == EMISSION_RUN)
-		    emission_state = EMISSION_STOP;
-		  accu_used = TRUE;
-		}
-	      else
-		g_closure_invoke (handler->closure,
-				  return_value,
-				  node->n_params + 1,
-				  instance_and_params,
-				  &ihint);
+	      g_closure_invoke (handler->closure,
+				return_accu,
+				node->n_params + 1,
+				instance_and_params,
+				&ihint);
+	      if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
+		  emission_state == EMISSION_RUN)
+		emission_state = EMISSION_STOP;
 	      G_LOCK (g_signal_mutex);
+	      return_value_altered = TRUE;
 	      
 	      tmp = emission_state == EMISSION_RUN ? handler->next : NULL;
 	    }
@@ -1572,27 +2039,16 @@ signal_emit_R (SignalNode   *node,
       emission_state = EMISSION_RUN;
       
       G_UNLOCK (g_signal_mutex);
-      if (accumulator)
-	{
-	  if (accu_used)
-	    g_value_reset (&accu);
-	  g_closure_invoke (class_closure,
-			    &accu,
-			    node->n_params + 1,
-			    instance_and_params,
-			    &ihint);
-          if (!accumulator (&ihint, return_value, &accu) &&
-	      emission_state == EMISSION_RUN)
-	    emission_state = EMISSION_STOP;
-	  accu_used = TRUE;
-	}
-      else
-	g_closure_invoke (class_closure,
-			  return_value,
-			  node->n_params + 1,
-			  instance_and_params,
-			  &ihint);
+      g_closure_invoke (class_closure,
+			return_accu,
+			node->n_params + 1,
+			instance_and_params,
+			&ihint);
+      if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
+	  emission_state == EMISSION_RUN)
+	emission_state = EMISSION_STOP;
       G_LOCK (g_signal_mutex);
+      return_value_altered = TRUE;
       
       if (emission_state == EMISSION_STOP)
 	goto EMIT_CLEANUP;
@@ -1613,27 +2069,16 @@ signal_emit_R (SignalNode   *node,
 	  if (handler->after && !handler->block_count && (!handler->detail || handler->detail == detail))
 	    {
 	      G_UNLOCK (g_signal_mutex);
-              if (accumulator)
-		{
-		  if (accu_used)
-		    g_value_reset (&accu);
-		  g_closure_invoke (handler->closure,
-				    &accu,
-				    node->n_params + 1,
-				    instance_and_params,
-				    &ihint);
-		  if (!accumulator (&ihint, return_value, &accu) &&
-		      emission_state == EMISSION_RUN)
-		    emission_state = EMISSION_STOP;
-		  accu_used = TRUE;
-		}
-	      else
-		g_closure_invoke (handler->closure,
-				  return_value,
-				  node->n_params + 1,
-				  instance_and_params,
-				  &ihint);
+	      g_closure_invoke (handler->closure,
+				return_accu,
+				node->n_params + 1,
+				instance_and_params,
+				&ihint);
+	      if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
+		  emission_state == EMISSION_RUN)
+		emission_state = EMISSION_STOP;
 	      G_LOCK (g_signal_mutex);
+	      return_value_altered = TRUE;
 	      
 	      tmp = emission_state == EMISSION_RUN ? handler->next : NULL;
 	    }
@@ -1664,15 +2109,10 @@ signal_emit_R (SignalNode   *node,
       emission_state = EMISSION_STOP;
       
       G_UNLOCK (g_signal_mutex);
-      if (node->return_type != G_TYPE_NONE)
+      if (node->return_type != G_TYPE_NONE && !accumulator)
 	{
-	  if (!accumulator)
-	    {
-	      g_value_init (&accu, node->return_type);
-	      need_unset = TRUE;
-	    }
-	  else if (accu_used)
-	    g_value_reset (&accu);
+	  g_value_init (&accu, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
+	  need_unset = TRUE;
 	}
       g_closure_invoke (class_closure,
 			node->return_type != G_TYPE_NONE ? &accu : NULL,
@@ -1692,9 +2132,13 @@ signal_emit_R (SignalNode   *node,
   
   emission_pop ((node->flags & G_SIGNAL_NO_RECURSE) ? &g_restart_emissions : &g_recursive_emissions, &emission_state);
   if (accumulator)
-    {
-      G_UNLOCK (g_signal_mutex);
-      g_value_unset (&accu);
-      G_LOCK (g_signal_mutex);
-    }
+    g_value_unset (&accu);
+  
+  return return_value_altered;
 }
+
+
+/* --- compile standard marshallers --- */
+#include	"gobject.h"
+#include	"genums.h"
+#include        "gmarshal.c"
