@@ -1,5 +1,5 @@
 /* GObject - GLib Type, Object, Parameter and Signal Library
- * Copyright (C) 2000 Red Hat, Inc.
+ * Copyright (C) 2000-2001 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,12 +23,9 @@
 #include	<gobject/gclosure.h>
 #include	<gobject/gvalue.h>
 #include	<gobject/gparam.h>
+#include	<gobject/gmarshal.h>
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 /* --- typedefs --- */
 typedef struct _GSignalQuery		 GSignalQuery;
@@ -36,10 +33,12 @@ typedef struct _GSignalInvocationHint	 GSignalInvocationHint;
 typedef GClosureMarshal			 GSignalCMarshaller;
 typedef gboolean (*GSignalEmissionHook) (GSignalInvocationHint *ihint,
 					 guint			n_param_values,
-					 const GValue	       *param_values);
+					 const GValue	       *param_values,
+					 gpointer		data);
 typedef gboolean (*GSignalAccumulator)	(GSignalInvocationHint *ihint,
 					 GValue		       *return_accu,
-					 const GValue	       *return_value);
+					 const GValue	       *handler_return,
+					 gpointer               data);
 
 
 /* --- run & match types --- */
@@ -52,8 +51,8 @@ typedef enum
   G_SIGNAL_DETAILED	= 1 << 4,
   G_SIGNAL_ACTION	= 1 << 5,
   G_SIGNAL_NO_HOOKS	= 1 << 6
-#define G_SIGNAL_FLAGS_MASK  0x7f
 } GSignalFlags;
+#define G_SIGNAL_FLAGS_MASK  0x7f
 typedef enum
 {
   G_SIGNAL_MATCH_ID	   = 1 << 0,
@@ -62,8 +61,9 @@ typedef enum
   G_SIGNAL_MATCH_FUNC	   = 1 << 3,
   G_SIGNAL_MATCH_DATA	   = 1 << 4,
   G_SIGNAL_MATCH_UNBLOCKED = 1 << 5
-#define G_SIGNAL_MATCH_MASK  0x3f
 } GSignalMatchType;
+#define G_SIGNAL_MATCH_MASK  0x3f
+#define	G_SIGNAL_TYPE_STATIC_SCOPE (G_TYPE_FLAG_RESERVED_ID_BIT)
 
 
 /* --- signal information --- */
@@ -77,44 +77,85 @@ struct _GSignalQuery
 {
   guint		signal_id;
   const gchar  *signal_name;
-  GType		itype;
+  GType		itype;	     /* mangled with G_SIGNAL_TYPE_STATIC_SCOPE flag */
   GSignalFlags	signal_flags;
-  GType		return_type;
+  GType		return_type; /* mangled with G_SIGNAL_TYPE_STATIC_SCOPE flag */
   guint		n_params;
   const GType  *param_types;
 };
 
 
 /* --- signals --- */
-guint	g_signal_newv			      (const gchar	 *signal_name,
-					       GType		  itype,
-					       GSignalFlags	   signal_flags,
-					       GClosure		 *class_closure,
-					       GSignalAccumulator accumulator,
-					       GSignalCMarshaller c_marshaller,
-					       GType		  return_type,
-					       guint		  n_params,
-					       GType		 *param_types);
-void	g_signal_emitv			      (const GValue	 *instance_and_params,
-					       guint		  signal_id,
-					       GQuark		  detail,
-					       GValue		 *return_value);
-guint	g_signal_lookup			      (const gchar	 *name,
-					       GType		  itype);
-gchar*	g_signal_name			      (guint		  signal_id);
-void	g_signal_query			      (guint		  signal_id,
-					       GSignalQuery	 *query);
-guint*	g_signal_list_ids		      (GType		  itype,
-					       guint		 *n_ids);
+guint                 g_signal_newv         (const gchar        *signal_name,
+					     GType               itype,
+					     GSignalFlags        signal_flags,
+					     GClosure           *class_closure,
+					     GSignalAccumulator	 accumulator,
+					     gpointer		 accu_data,
+					     GSignalCMarshaller  c_marshaller,
+					     GType               return_type,
+					     guint               n_params,
+					     GType              *param_types);
+guint                 g_signal_new_valist   (const gchar        *signal_name,
+					     GType               itype,
+					     GSignalFlags        signal_flags,
+					     GClosure           *class_closure,
+					     GSignalAccumulator	 accumulator,
+					     gpointer		 accu_data,
+					     GSignalCMarshaller  c_marshaller,
+					     GType               return_type,
+					     guint               n_params,
+					     va_list             args);
+guint                 g_signal_newc         (const gchar        *signal_name,
+					     GType               itype,
+					     GSignalFlags        signal_flags,
+					     guint               class_offset,
+					     GSignalAccumulator	 accumulator,
+					     gpointer		 accu_data,
+					     GSignalCMarshaller  c_marshaller,
+					     GType               return_type,
+					     guint               n_params,
+					     ...);
+void                  g_signal_emitv        (const GValue       *instance_and_params,
+					     guint               signal_id,
+					     GQuark              detail,
+					     GValue             *return_value);
+void                  g_signal_emit_valist  (gpointer            instance,
+					     guint               signal_id,
+					     GQuark              detail,
+					     va_list             var_args);
+void                  g_signal_emit         (gpointer            instance,
+					     guint               signal_id,
+					     GQuark              detail,
+					     ...);
+void                  g_signal_emit_by_name (gpointer            instance,
+					     const gchar        *detailed_signal,
+					     ...);
+guint                 g_signal_lookup       (const gchar        *name,
+					     GType               itype);
+G_CONST_RETURN gchar* g_signal_name         (guint               signal_id);
+void                  g_signal_query        (guint               signal_id,
+					     GSignalQuery       *query);
+guint*                g_signal_list_ids     (GType               itype,
+					     guint              *n_ids);
+gboolean	      g_signal_parse_name   (const gchar	*detailed_signal,
+					     GType		 itype,
+					     guint		*signal_id_p,
+					     GQuark		*detail_p,
+					     gboolean		 force_detail_quark);
+
 
 /* --- signal emissions --- */
-void	g_signal_stop_emission		      (gpointer		  instance,
-					       guint		  signal_id,
-					       GQuark		  detail);
-guint	g_signal_add_emission_hook_full	      (guint		  signal_id,
-					       GClosure		 *closure);
-void	g_signal_remove_emission_hook	      (guint		  signal_id,
-					       guint		  hook_id);
+void	g_signal_stop_emission		    (gpointer		  instance,
+					     guint		  signal_id,
+					     GQuark		  detail);
+guint	g_signal_add_emission_hook	    (guint		  signal_id,
+					     GQuark		  quark,
+					     GSignalEmissionHook  hook_func,
+					     gpointer	       	  hook_data,
+					     GDestroyNotify	  data_destroy);
+void	g_signal_remove_emission_hook	    (guint		  signal_id,
+					     guint		  hook_id);
 
 
 /* --- signal handlers --- */
@@ -127,6 +168,17 @@ guint	 g_signal_connect_closure_by_id	      (gpointer		  instance,
 					       GQuark		  detail,
 					       GClosure		 *closure,
 					       gboolean		  after);
+guint	 g_signal_connect_closure	      (gpointer		  instance,
+					       const gchar       *detailed_signal,
+					       GClosure		 *closure,
+					       gboolean		  after);
+guint	 g_signal_connect_data		      (gpointer		  instance,
+					       const gchar	 *detailed_signal,
+					       GCallback	  c_handler,
+					       gpointer		  data,
+					       GClosureNotify	  destroy_data,
+					       gboolean		  swapped,
+					       gboolean		  after);
 void	 g_signal_handler_block		      (gpointer		  instance,
 					       guint		  handler_id);
 void	 g_signal_handler_unblock	      (gpointer		  instance,
@@ -163,18 +215,14 @@ guint	 g_signal_handlers_disconnect_matched (gpointer		  instance,
 					       gpointer		  data);
 
 
+/* --- convenience --- */
+#define g_signal_connectc(instance, detailed_signal, c_handler, data, swapped) \
+    g_signal_connect_data ((instance), (detailed_signal), (c_handler), (data), NULL, (swapped), FALSE)
+
 /*< private >*/
-gboolean g_signal_parse_name		      (const gchar	 *detailed_signal,
-					       GType		  itype,
-					       guint		 *signal_id_p,
-					       GQuark		 *detail_p,
-					       gboolean		  force_detail_quark);
 void	 g_signal_handlers_destroy	      (gpointer		  instance);
 void	 _g_signals_destroy		      (GType		  itype);
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __G_SIGNAL_H__ */
