@@ -1,4 +1,4 @@
-/* gspawn.c - Process launching
+/* gspawn-win32.c - Process launching on Win32
  *
  *  Copyright 2000 Red Hat, Inc.
  *
@@ -53,11 +53,7 @@
 #include <process.h>
 #include <direct.h>
 
-#ifdef _
-#warning "FIXME remove gettext hack"
-#endif
-
-#define _(x) x
+#include "glibintl.h"
 
 #ifdef G_SPAWN_WIN32_DEBUG
   static int debug = 1;
@@ -69,10 +65,12 @@
     G_STMT_START					\
       {							\
 	if (debug == -1)				\
-	  if (getenv ("G_SPAWN_WIN32_DEBUG") != NULL)	\
-	    debug = 1;					\
-	  else						\
-	    debug = 0;					\
+	  {						\
+	    if (getenv ("G_SPAWN_WIN32_DEBUG") != NULL)	\
+	      debug = 1;				\
+	    else					\
+	      debug = 0;				\
+	  }						\
       }							\
     G_STMT_END
 #endif
@@ -986,208 +984,4 @@ make_pipe (gint     p[2],
     return TRUE;
 }
 
-#else /* GSPAWN_HELPER */
-
-static void
-write_err_and_exit (gint fd,
-		    gint msg)
-{
-  gint en = errno;
-  
-  write (fd, &msg, sizeof(msg));
-  write (fd, &en, sizeof(en));
-  
-  _exit (1);
-}
-
-static void
-write_no_error (gint fd)
-{
-  gint msg = CHILD_NO_ERROR;
-  gint en = 0;
-
-  write (fd, &msg, sizeof(msg));
-  write (fd, &en, sizeof(en));
-}
-
-#ifdef __GNUC__
-#  ifndef _stdcall
-#    define _stdcall  __attribute__((stdcall))
-#  endif
-#endif
-
-/* We build gspawn-win32-helper.exe as a Windows GUI application
- * to avoid any temporarily flashing console windows in case
- * the gspawn function is invoked by a GUI program. Thus, no main()
- * but a WinMain(). We do, however, still use argc and argv tucked
- * away in the global __argc and __argv by the C runtime startup code.
- */
-
-int _stdcall
-WinMain (struct HINSTANCE__ *hInstance,
-	 struct HINSTANCE__ *hPrevInstance,
-	 char               *lpszCmdLine,
-	 int                 nCmdShow)
-{
-  int child_err_report_fd;
-  int i;
-  int fd;
-  int mode;
-  GString *debugstring;
-
-  SETUP_DEBUG();
-
-  if (debug)
-    {
-      debugstring = g_string_new ("");
-
-      g_string_append (debugstring,
-		       g_strdup_printf ("g-spawn-win32-helper: "
-					"argc = %d, argv: ",
-					__argc));
-      for (i = 0; i < __argc; i++)
-	{
-	  if (i > 0)
-	    g_string_append (debugstring, " ");
-	  g_string_append (debugstring, __argv[i]);
-	}
-      
-      MessageBox (NULL, debugstring->str, "gspawn-win32-helper", 0);
-    }
-
-  g_assert (__argc >= ARG_COUNT);
-
-  /* argv[ARG_CHILD_ERR_REPORT] is the file descriptor onto which
-   * write error messages.
-   */
-  child_err_report_fd = atoi (__argv[ARG_CHILD_ERR_REPORT]);
-
-  /* argv[ARG_STDIN..ARG_STDERR] are the file descriptors that should
-   * be dup2'd to stdin, stdout and stderr, '-' if the corresponding
-   * std* should be let alone, and 'z' if it should be connected to
-   * the bit bucket NUL:.
-   */
-  if (__argv[ARG_STDIN][0] == '-')
-    ; /* Nothing */
-  else if (__argv[ARG_STDIN][0] == 'z')
-    {
-      fd = open ("NUL:", O_RDONLY);
-      if (fd != 0)
-	{
-	  dup2 (fd, 0);
-	  close (fd);
-	}
-    }
-  else
-    {
-      fd = atoi (__argv[ARG_STDIN]);
-      if (fd != 0)
-	{
-	  dup2 (fd, 0);
-	  close (fd);
-	}
-    }
-
-  if (__argv[ARG_STDOUT][0] == '-')
-    ; /* Nothing */
-  else if (__argv[ARG_STDOUT][0] == 'z')
-    {
-      fd = open ("NUL:", O_WRONLY);
-      if (fd != 1)
-	{
-	  dup2 (fd, 1);
-	  close (fd);
-	}
-    }
-  else
-    {
-      fd = atoi (__argv[ARG_STDOUT]);
-      if (fd != 1)
-	{
-	  dup2 (fd, 1);
-	  close (fd);
-	}
-    }
-
-  if (__argv[ARG_STDERR][0] == '-')
-    ; /* Nothing */
-  else if (__argv[ARG_STDERR][0] == 'z')
-    {
-      fd = open ("NUL:", O_WRONLY);
-      if (fd != 2)
-	{
-	  dup2 (fd, 2);
-	  close (fd);
-	}
-    }
-  else
-    {
-      fd = atoi (__argv[ARG_STDERR]);
-      if (fd != 2)
-	{
-	  dup2 (fd, 2);
-	  close (fd);
-	}
-    }
-
-  /* __argv[ARG_WORKING_DIRECTORY] is the directory in which to run the
-   * process.  If "-", don't change directory.
-   */
-  if (__argv[ARG_WORKING_DIRECTORY][0] == '-' &&
-      __argv[ARG_WORKING_DIRECTORY][1] == 0)
-    ; /* Nothing */
-  else if (chdir (__argv[ARG_WORKING_DIRECTORY]) < 0)
-    write_err_and_exit (child_err_report_fd,
-			CHILD_CHDIR_FAILED);
-
-  /* __argv[ARG_CLOSE_DESCRIPTORS] is "y" if file descriptors from 3
-   *  upwards should be closed
-   */
-
-  if (__argv[ARG_CLOSE_DESCRIPTORS][0] == 'y')
-    for (i = 3; i < 1000; i++)	/* FIXME real limit? */
-      if (i != child_err_report_fd)
-	close (i);
-
-  /* __argv[ARG_WAIT] is "w" to wait for the program to exit */
-
-  if (__argv[ARG_WAIT][0] == 'w')
-    mode = P_WAIT;
-  else
-    mode = P_NOWAIT;
-
-  /* __argv[ARG_USE_PATH] is "y" to use PATH, otherwise not */
-
-  /* __argv[ARG_PROGRAM] is program file to run,
-   * __argv[ARG_PROGRAM+1]... is its __argv.
-   */
-
-  if (debug)
-    {
-      debugstring = g_string_new ("");
-      g_string_append (debugstring,
-		       g_strdup_printf ("calling %s on program %s, __argv: ",
-					(__argv[ARG_USE_PATH][0] == 'y' ?
-					 "spawnvp" : "spawnv"),
-					__argv[ARG_PROGRAM]));
-      i = ARG_PROGRAM+1;
-      while (__argv[i])
-	g_string_append (debugstring, __argv[i++]);
-      MessageBox (NULL, debugstring->str, "gspawn-win32-helper", 0);
-    }
-
-  if (__argv[ARG_USE_PATH][0] == 'y')
-    {
-      if (spawnvp (mode, __argv[ARG_PROGRAM], __argv+ARG_PROGRAM) < 0)
-	write_err_and_exit (child_err_report_fd, CHILD_SPAWN_FAILED);
-    }
-  else
-    {
-      if (spawnv (mode, __argv[ARG_PROGRAM], __argv+ARG_PROGRAM) < 0)
-	write_err_and_exit (child_err_report_fd, CHILD_SPAWN_FAILED);
-    }
-
-  return 0;
-}
-
-#endif /* GSPAWN_HELPER */
+#endif /* !GSPAWN_HELPER */
