@@ -30,8 +30,11 @@ G_BEGIN_DECLS
 typedef guint32 gunichar;
 typedef guint16 gunichar2;
 
-/* These are the possible character classifications.  */
-typedef enum {
+/* These are the possible character classifications.
+ * See http://www.unicode.org/Public/UNIDATA/UnicodeData.html
+ */
+typedef enum
+{
   G_UNICODE_CONTROL,
   G_UNICODE_FORMAT,
   G_UNICODE_UNASSIGNED,
@@ -64,9 +67,47 @@ typedef enum {
   G_UNICODE_SPACE_SEPARATOR
 } GUnicodeType;
 
+/* These are the possible line break classifications.
+ * See http://www.unicode.org/unicode/reports/tr14/
+ */
+typedef enum
+{
+  G_UNICODE_BREAK_MANDATORY,
+  G_UNICODE_BREAK_CARRIAGE_RETURN,
+  G_UNICODE_BREAK_LINE_FEED,
+  G_UNICODE_BREAK_COMBINING_MARK,
+  G_UNICODE_BREAK_SURROGATE,
+  G_UNICODE_BREAK_ZERO_WIDTH_SPACE,
+  G_UNICODE_BREAK_INSEPARABLE,
+  G_UNICODE_BREAK_NON_BREAKING_GLUE,
+  G_UNICODE_BREAK_CONTINGENT,
+  G_UNICODE_BREAK_SPACE,
+  G_UNICODE_BREAK_AFTER,
+  G_UNICODE_BREAK_BEFORE,
+  G_UNICODE_BREAK_BEFORE_AND_AFTER,
+  G_UNICODE_BREAK_HYPHEN,
+  G_UNICODE_BREAK_NON_STARTER,
+  G_UNICODE_BREAK_OPEN_PUNCTUATION,
+  G_UNICODE_BREAK_CLOSE_PUNCTUATION,
+  G_UNICODE_BREAK_QUOTATION,
+  G_UNICODE_BREAK_EXCLAMATION,
+  G_UNICODE_BREAK_IDEOGRAPHIC,
+  G_UNICODE_BREAK_NUMERIC,
+  G_UNICODE_BREAK_INFIX_SEPARATOR,
+  G_UNICODE_BREAK_SYMBOL,
+  G_UNICODE_BREAK_ALPHABETIC,
+  G_UNICODE_BREAK_PREFIX,
+  G_UNICODE_BREAK_POSTFIX,
+  G_UNICODE_BREAK_COMPLEX_CONTEXT,
+  G_UNICODE_BREAK_AMBIGUOUS,
+  G_UNICODE_BREAK_UNKNOWN
+} GUnicodeBreakType;
+
 /* Returns TRUE if current locale uses UTF-8 charset.  If CHARSET is
  * not null, sets *CHARSET to the name of the current locale's
- * charset.  This value is statically allocated.
+ * charset.  This value is statically allocated, and should be copied
+ * in case the locale's charset will be changed later using setlocale()
+ * or in some other way.
  */
 gboolean g_get_charset (char **charset);
 
@@ -102,6 +143,8 @@ gint g_unichar_xdigit_value (gunichar c) G_GNUC_CONST;
 /* Return the Unicode character type of a given character.  */
 GUnicodeType g_unichar_type (gunichar c) G_GNUC_CONST;
 
+/* Return the line break property for a given character */
+GUnicodeBreakType g_unichar_break_type (gunichar c) G_GNUC_CONST;
 
 
 /* Compute canonical ordering of a string in-place.  This rearranges
@@ -118,63 +161,72 @@ gunichar *g_unicode_canonical_decomposition (gunichar  ch,
 
 /* Array of skip-bytes-per-initial character.
  * We prefix variable declarations so they can
- * properly get exported in windows dlls.
+ * properly get exported in Windows DLLs.
  */
-#ifndef GLIB_VAR
-#  ifdef G_OS_WIN32
-#    ifdef GLIB_COMPILATION
-#      define GLIB_VAR __declspec(dllexport)
-#    else /* !GLIB_COMPILATION */
-#      define GLIB_VAR extern __declspec(dllimport)
-#    endif /* !GLIB_COMPILATION */
-#  else /* !G_OS_WIN32 */
-#    define GLIB_VAR extern
-#  endif /* !G_OS_WIN32 */
-#endif /* !GLIB_VAR */
-
 GLIB_VAR char g_utf8_skip[256];
 
 #define g_utf8_next_char(p) (char *)((p) + g_utf8_skip[*(guchar *)(p)])
 
 gunichar g_utf8_get_char          (const gchar *p);
-gchar *  g_utf8_offset_to_pointer  (const gchar *str,
-				    gint         offset);
+gchar*   g_utf8_offset_to_pointer (const gchar *str,
+                                   gint         offset);
 gint     g_utf8_pointer_to_offset (const gchar *str,
 				   const gchar *pos);
-gchar *  g_utf8_prev_char         (const gchar *p);
-gchar *  g_utf8_find_next_char    (const gchar *p,
+gchar*   g_utf8_prev_char         (const gchar *p);
+gchar*   g_utf8_find_next_char    (const gchar *p,
 				   const gchar *end);
-gchar *  g_utf8_find_prev_char    (const gchar *str,
+gchar*   g_utf8_find_prev_char    (const gchar *str,
 				   const gchar *p);
 
 gint g_utf8_strlen (const gchar *p,
 		    gint         max);
 
 /* Copies n characters from src to dest */
-gchar *g_utf8_strncpy (gchar       *dest,
+gchar* g_utf8_strncpy (gchar       *dest,
 		       const gchar *src,
 		       size_t       n);
 
 /* Find the UTF-8 character corresponding to ch, in string p. These
    functions are equivalants to strchr and strrchr */
 
-gchar *g_utf8_strchr  (const gchar *p,
+gchar* g_utf8_strchr  (const gchar *p,
 		       gunichar     c);
-gchar *g_utf8_strrchr (const gchar *p,
+gchar* g_utf8_strrchr (const gchar *p,
 		       gunichar     c);
 
-gunichar2 *g_utf8_to_utf16 (const gchar     *str,
-			    gint             len);
-gunichar * g_utf8_to_ucs4  (const gchar     *str,
-			    gint             len);
-gunichar * g_utf16_to_ucs4 (const gunichar2 *str,
-			    gint             len);
-gchar *    g_utf16_to_utf8 (const gunichar2 *str,
-			    gint             len);
-gunichar * g_ucs4_to_utf16 (const gunichar  *str,
-			    gint             len);
-gchar *    g_ucs4_to_utf8  (const gunichar  *str,
-			    gint             len);
+gunichar2 *g_utf8_to_utf16     (const gchar      *str,
+				gint              len,
+				gint             *items_read,
+				gint             *items_written,
+				GError          **error);
+gunichar * g_utf8_to_ucs4      (const gchar      *str,
+				gint              len,
+				gint             *items_read,
+				gint             *items_written,
+				GError          **error);
+gunichar * g_utf8_to_ucs4_fast (const gchar      *str,
+				gint              len,
+				gint             *items_written);
+gunichar * g_utf16_to_ucs4     (const gunichar2  *str,
+				gint              len,
+				gint             *items_read,
+				gint             *items_written,
+				GError          **error);
+gchar*     g_utf16_to_utf8     (const gunichar2  *str,
+				gint              len,
+				gint             *items_read,
+				gint             *items_written,
+				GError          **error);
+gunichar2 *g_ucs4_to_utf16     (const gunichar   *str,
+				gint              len,
+				gint             *items_read,
+				gint             *items_written,
+				GError          **error);
+gchar*     g_ucs4_to_utf8      (const gunichar   *str,
+				gint              len,
+				gint             *items_read,
+				gint             *items_written,
+				GError          **error);
 
 /* Convert a single character into UTF-8. outbuf must have at
  * least 6 bytes of space. Returns the number of bytes in the
@@ -191,6 +243,9 @@ gboolean g_utf8_validate (const gchar  *str,
                           gint          max_len,
                           const gchar **end);
 
+/* Validate a Unicode character */
+gboolean g_unichar_validate (gunichar ch);
+
 G_END_DECLS
 
 #endif /* __G_UNICODE_H__ */
